/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_BLOCKING_RECV_SETUP_H
#define _EXS_BLOCKING_RECV_SETUP_H

#include "../result.h"

#define BLOCKING_RECV_TESTS 8

static char *all_blocking_recv_tests[BLOCKING_RECV_TESTS] = {
	"1  : EPERM                   : exs_blocking_recv(fd, recv_buf, RECV_BUFSIZE, flags, mh) [no exs_init]",
	"2  : EBADF                   : exs_blocking_recv(BAD_FD, recv_buf, RECV_BUFSIZE, flags, mh)",
	"3  : ENOTCONN                : exs_blocking_recv(NOT_CONNECTED_FD, recv_buf, RECV_BUFSIZE, flags, mh)",
	"4  : EINVAL                  : exs_blocking_recv(fd, NULL_BUFFER, RECV_BUFSIZE, flags, mh)",
	"5  : EINVAL                  : exs_blocking_recv(fd, recv_buf, 0, flags, mh) [length < 1]",
	"6  : EINVAL                  : exs_blocking_recv(fd, recv_buf, MAX_LENGTH+1, flags, mh) [length > RDMA channel]",
	"7  : EOPNOTSUPP              : exs_blocking_recv(fd, recv_buf, RECV_BUFSIZE, BAD_FLAG, mh)",
	"8  : OK_EXS_BLOCKING_RECV    : exs_blocking_recv(fd, recv_buf, RECV_BUFSIZE, flags, mh)"
};

static int list_blocking_recv_tests(int select)
{
	if (select < 0 || select > BLOCKING_RECV_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			BLOCKING_RECV_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < BLOCKING_RECV_TESTS; i++) {
			fprintf(stdout, "%s\n",
				all_blocking_recv_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n",
			all_blocking_recv_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_blocking_recv(param * p)
{
	char *name = "ok_exs_blocking_recv";

	int ret =
	    exs_blocking_recv(p->recv_fd, p->recv_buf, p->RECV_BUFSIZE,
			      p->flags, p->mh);

	if (ret < p->RECV_BUFSIZE) {
		print_error(name);
		return -1;
	}
	return ret;

}

#endif
