/*
 * Copyright 2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef EXS_GETSOCKOPT_SETUP_H
#define EXS_GETSOCKOPT_SETUP_H

#define GETSOCKOPT_TESTS 11

static char *all_getsockopt_tests[GETSOCKOPT_TESTS] = {
	"1  : EPERM                   : exs_getsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &v, &len) [no exs_init]",
	"2  : EBADF                   : exs_getsockopt(BAD_FD, SOL_SOCKET, SO_REUSEADDR, &v, &len)",
	"3  : EOPNOTSUPP              : exs_getsockopt(fd, BAD_LEVEL, SO_REUSEADDR, &v, &len)",
	"4  : ENOPROTOOPT             : exs_getsockopt(fd, SOL_SOCKET, BAD_OPTNAME, &v, &len)",
	"5  : ENOPROTOOPT             : exs_getsockopt(fd, SOL_EXS, BAD_OPTNAME, &v, &len)",
	"6  : OK optval=0 optlen=4    : exs_getsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &v, &len)",
	"7  : OK optval=MAX optlen=4  : exs_getsockopt(fd, SOL_EXS, SO_INLINE, &v, &len)",
	"8  : OK optval=0 optlen=4    : exs_getsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &v, &TOO_SMALL_OPTLEN)",
	"9  : OK optval=0 optlen=4    : exs_getsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &v, &TOO_LARGE_OPTLEN)",
	"10 : OK optval=MAX optlen=4  : exs_getsockopt(fd, SOL_EXS, SO_INLINE, &v, &TOO_SMALL_OPTLEN)",
	"11 : OK optval=MAX optlen=4  : exs_getsockopt(fd, SOL_EXS, SO_INLINE, &v, &TOO_LARGE_OPTLEN)",
};

static int list_getsockopt_tests(int select)
{
	if (select < 0 || select > GETSOCKOPT_TESTS) {
		printf("Invalid -t test number (1 - %d)\n", GETSOCKOPT_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < GETSOCKOPT_TESTS; i++)
			if (strcmp(all_getsockopt_tests[i], "") != 0)
				printf("%s\n", all_getsockopt_tests[i]);
	} else {
		printf("%s\n", all_getsockopt_tests[select - 1]);
	}

	exit(0);
} /* list_getsockopt_tests */

#endif
