/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_QDEQUEUE_SETUP_H
#define _EXS_QDEQUEUE_SETUP_H

#include "../result.h"

#define QDEQUEUE_TESTS 5

static char *all_qdequeue_tests[QDEQUEUE_TESTS] = {
	"1  : EPERM                   : exs_qdequeue(qh, evtvec, evtvec_cnt, to) [no exs_init]",
	"2  : EINVAL                  : exs_qdequeue(INVALID_QH, evtvec, evtvec_cent, to)",
	"3  : EINVAL                  : exs_qdequeue(qh, evtvec, ZERO_EVTVEC_CNT, to) [evtvec_cnt < 1]",
	"",
	"5  : OK_EXS_QDEQUEUE         : exs_qdequeue(qh, evtvec, evtvec_cent, to)"
};

static int list_qdequeue_tests(int select)
{
	if (select < 0 || select > QDEQUEUE_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			QDEQUEUE_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < QDEQUEUE_TESTS; i++) {
			if (strcmp(all_qdequeue_tests[i], "") != 0)
				fprintf(stdout, "%s\n",
					all_qdequeue_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_qdequeue_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_qdequeue(param * p)
{
	char *name = "ok_exs_dequeue";

	int ret =
	    exs_qdequeue(p->qh, &p->dequeue_event, p->evtvec_cnt, NULL);

	if (p->mode > 0) {
		if( p->dequeue_event.exs_evt_type == EXS_EVT_ACCEPT){
			p->recv_fd =
				p->dequeue_event.exs_evt_union.exs_evt_accept.
				exs_evt_new_socket;
		}
	}

	if (ret < 0) {
		print_error(name);
		return -1;
	}
	return 0;
}

#endif
