/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_READ_SETUP_H
#define _EXS_READ_SETUP_H

#include "../result.h"

#define READ_TESTS 7

static char *all_read_tests[READ_TESTS] = {
	"1  : EPERM                   : exs_read(fd, recv_buf, RECV_BUFSIZE) [no exs_init]",
	"2  : EBADF                   : exs_read(BAD_FD, recv_buf, RECV_BUFSIZE)",
	"3  : ENOTCONN                : exs_read(NOT_CONNECTED_FD, recv_buf, RECV_BUFSIZE)",
	"4  : EINVAL                  : exs_read(fd, INVALID_QHANDLE, RECV_BUFSIZE)",
	"5  : EINVAL                  : exs_read(fd, recv_buf, ZERO_LENGTH) [length < 1 ]",
	"6  : EINVAL                  : exs_read(fd, recv_buf, MAX_LENGTH) [length > rdma channel]",
	"7  : OK_EXS_READ             : exs_read(fd, recv_buf, RECV_BUFSIZE)"
};

static int list_read_tests(int select)
{
	if (select < 0 || select > READ_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			READ_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < READ_TESTS; i++) {
			if (strcmp(all_read_tests[i], "") != 0)
				fprintf(stdout, "%s\n", all_read_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_read_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_read(param * p)
{
	char *name = "ok_exs_read";

	int ret = exs_read(p->recv_fd, p->recv_buf, p->RECV_BUFSIZE);

	if (ret < p->RECV_BUFSIZE) {
		fprintf(stdout, "errno: %s\n", strerror(errno));
		print_error(name);
		return -1;
	}

	return ret;

}

#endif
