/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_SOCKET_PARAMS_H
#define _EXS_SOCKET_PARAMS_H

#include "../result.h"

#define PASS 0

#define NUM_PARAM 4
#define NUM_TESTS 15

static int exs_socket_params[NUM_TESTS][NUM_PARAM] = {
	{-1, SOCK_STREAM, IPPROTO_TCP, EAFNOSUPPORT},	//[1]
	{PF_INET, -1, IPPROTO_TCP, ESOCKTNOSUPPORT},	//[2]
	{PF_INET, -1, IPPROTO_SCTP, ESOCKTNOSUPPORT},	//[3]
	{PF_INET, SOCK_STREAM, -1, EPROTONOSUPPORT},	//[4]

	{PF_INET, SOCK_STREAM, IPPROTO_TCP, PASS},	//[5]
	{PF_INET, SOCK_STREAM, -1, EPROTONOSUPPORT},	//[6]
	{PF_INET, SOCK_STREAM, IPPROTO_SCTP, EPROTONOSUPPORT},	//[7]

	{PF_INET6, SOCK_STREAM, IPPROTO_TCP, PASS},	//[8]
	{PF_INET6, SOCK_STREAM, -1, EPROTONOSUPPORT},	//[9]
	{PF_INET6, SOCK_STREAM, IPPROTO_SCTP, EPROTONOSUPPORT},	//[10]

	{PF_INET6, SOCK_SEQPACKET, IPPROTO_SCTP, PASS},	//[11]
	{PF_INET6, SOCK_SEQPACKET, -1, EPROTONOSUPPORT},	//[12]
	{PF_INET6, SOCK_SEQPACKET, IPPROTO_TCP, EPROTONOSUPPORT},	//[13]
	{PF_INET6, SOCK_SEQPACKET, IPPROTO_SCTP, PASS},	//[14]
	{PF_INET6, SOCK_SEQPACKET, -1, EPROTONOSUPPORT},	//[15]
};

static char *exs_socket_params_value[][NUM_TESTS] = {
	{"EAFNOSUPPORT"},	//[1]
	{"ESOCKTNOSUPPORT"},	//[2]
	{"ESOCKTNOSUPPORT"},	//[3]
	{"EPROTONOSUPPORT"},	//[4]

	{"PASS"},		//[5]
	{"EPROTONOSUPPORT"},	//[6]
	{"EPROTONOSUPPORT"},	//[7]

	{"PASS"},		//[8]
	{"EPROTONOSUPPORT"},	//[9]
	{"EPROTONOSUPPORT"},	//[10]

	{"PASS"},		//[11]
	{"EPROTONOSUPPORT"},	//[12]
	{"EPROTONOSUPPORT"},	//[13]
	{"PASS"},		//[14]
	{"EPROTONOSUPPORT"},	//[15]
};

static char *exs_socket_params_string[][NUM_TESTS] = {
	{"exs_socket(-1,       SOCK_STREAM,      IPPROTO_TCP)"},	//[1]
	{"exs_socket(PF_INET,           -1,      IPPROTO_TCP)"},	//[2]
	{"exs_socket(PF_INET,           -1,     IPPROTO_SCTP)"},	//[3]
	{"exs_socket(PF_INET,  SOCK_STREAM,               -1)"},	//[4]

	{"exs_socket(PF_INET,  SOCK_STREAM,      IPPROTO_TCP)"},	//[5]
	{"exs_socket(PF_INET,  SOCK_STREAM,               -1)"},	//[6]
	{"exs_socket(PF_INET,  SOCK_STREAM,     IPPROTO_SCTP)"},	//[7]

	{"exs_socket(PF_INET6, SOCK_STREAM,      IPPROTO_TCP)"},	//[8]
	{"exs_socket(PF_INET6, SOCK_STREAM,               -1)"},	//[9]
	{"exs_socket(PF_INET6, SOCK_STREAM,     IPPROTO_SCTP)"},	//[10]

	{"exs_socket(PF_INET6, SOCK_SEQPACKET,  IPPROTO_SCTP)"},	//[11]
	{"exs_socket(PF_INET6, SOCK_SEQPACKET,            -1)"},	//[12]
	{"exs_socket(PF_INET6, SOCK_SEQPACKET,   IPPROTO_TCP)"},	//[13]
	{"exs_socket(PF_INET6, SOCK_SEQPACKET,  IPPROTO_SCTP)"},	//[14]
	{"exs_socket(PF_INET6, SOCK_SEQPACKET,            -1)"},	//[15]
};

#endif
