/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_blocking_sendfile_setup.h"
#include "setup/exs_client_setup.h"
#include "setup/exs_server_setup.h"

void *test_blocking_sendfile(void *test_void_ptr)
{
	setup_fault_catcher(p, getpid());

	param *p = (param *) test_void_ptr;
	int test = p->test_number;

	if (p->mode > 0)
		return 0;

	int ret;

	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_blocking_sendfile_tests[test - 1];
		ret =
		    exs_blocking_sendfile(p->fd, p->sendvec,
					  p->sendvec_cnt, p->flags);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EPERM);
	}

	else if (test == 2) {
		p->test_value = "ENOTCONN";
		p->test_name = all_blocking_sendfile_tests[test - 1];

		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		if (p->mode > 0)
			if (ok_exs_qcreate(p) < 0)
				exit(EXIT_FAILURE);

		if (ok_exs_socket(p) < 0)
			exit(EXIT_FAILURE);

		ret =
		    exs_blocking_sendfile(p->fd, p->sendvec,
					  p->sendvec_cnt, p->flags);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    ENOTCONN);
	}

	else if (test == 3) {
		p->test_value = "EBADF";
		p->test_name = all_blocking_sendfile_tests[test - 1];
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		ret =
		    exs_blocking_sendfile(-1, p->sendvec, p->sendvec_cnt,
					  p->flags);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EBADF);
	}

	else if (test == 4) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			/** start exs_sendfile **/
			p->test_value = "ENODEV";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			int file_fd = p->fd;	//use socket instead of file

			sendvec.exs_xfv.exs_fdv.exs_fildes = file_fd;
			sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 0;
			sendvec.exs_xfv.exs_fdv.exs_length = 0;	//go until EOF
			sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_FDVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, ENODEV);
			/** end sendfile **/

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 5) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			/** start exs_sendfile **/
			p->test_value = "ENODEV";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			int file_fd = -1;

			sendvec.exs_xfv.exs_fdv.exs_fildes = file_fd;
			sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 0;
			sendvec.exs_xfv.exs_fdv.exs_length = 0;	//go until EOF
			sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_FDVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EINVAL);
			/** end sendfile **/

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 6) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			/** start exs_sendfile **/
			p->test_value = "ENODEV";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			int file_fd = p->fd;	//use socket instead of file

			sendvec.exs_xfv.exs_pathv.exs_path =
			    "temp/bad.txt";
			sendvec.exs_xfv.exs_pathv.exs_offset = (off_t) 0;
			sendvec.exs_xfv.exs_pathv.exs_length = 0;	//go until EOF
			sendvec.exs_xfv.exs_pathv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_PATHVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EINVAL);
			/** end sendfile **/

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 7) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			/** start exs_sendfile **/
			p->test_value = "ENODEV";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			int file_fd = open(exs_test_large_file, O_RDONLY);

			sendvec.exs_xfv.exs_fdv.exs_fildes = file_fd;
			sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 0;
			sendvec.exs_xfv.exs_fdv.exs_length = (size_t) 0;	//go until EOF
			sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_FDVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EFBIG);

			close(file_fd);
			/** end sendfile **/

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 8) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			/** start exs_sendfile **/
			p->test_value = "EINVAL";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			int file_fd = open(exs_test_small_file, O_RDONLY);

			sendvec.exs_xfv.exs_fdv.exs_fildes = file_fd;
			sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 0;
			sendvec.exs_xfv.exs_fdv.exs_length = 1025;	//go until EOF
			sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_FDVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EINVAL);
			close(file_fd);
			/** end sendfile **/

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 9) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			/** start exs_sendfile **/
			p->test_value = "EINVAL";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			int file_fd = open(exs_test_small_file, O_RDONLY);

			sendvec.exs_xfv.exs_fdv.exs_fildes = file_fd;
			sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 1025;
			sendvec.exs_xfv.exs_fdv.exs_length = 0;	//go until EOF
			sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_FDVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EINVAL);
			close(file_fd);
			/** end sendfile **/

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 10) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			if (p->mode == 0)
				close(fd);

			/** start exs_sendfile **/
			p->test_value = "EINVAL";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			ret =
			    exs_blocking_sendfile(p->fd, p->sendvec, 0,
						  p->flags);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EINVAL);
			/** end sendfile **/

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 11) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			/** start exs_sendfile **/
			p->test_value = "EOPNOTSUPP";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt, -1);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EOPNOTSUPP);
			/** end sendfile **/

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 12) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			if (p->mode == 0)
				close(fd);

			/** start exs_sendfile **/
			p->test_value = "EINVAL";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  EXS_BLOCK &
						  EXS_UNSIGNALED);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EINVAL);
			/** end sendfile **/

			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 13) {
		pid_t pid = fork();
		if (pid == 0) {
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);

			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);


			close(fd);

			int shut_ret =
			    exs_shutdown(p->fd, SHUT_WR, p->flags, p->qh,
					 p->ah);

			/** start exs_sendfile **/
			p->test_value = "EPIPE";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			int ret = ok_exs_blocking_sendfile(p);
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EPIPE);
			/** end sendfile **/


			fd = open(exs_test_fifo_2, O_WRONLY);
			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	else if (test == 14) {
		pid_t pid = fork();
		if (pid == 0) {
			/** blocking client **/

			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			p->test_value = "OK_EXS_BLOCKING_SENDFILE";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			int file_fd = open(exs_test_small_file, O_RDONLY);

			struct stat fileStat;
			fstat(file_fd, &fileStat);
			size_t length = fileStat.st_size;

			sendvec.exs_xfv.exs_fdv.exs_fildes = file_fd;
			sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 0;
			sendvec.exs_xfv.exs_fdv.exs_length = (size_t) 0;	//go until EOF
			sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_FDVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);

			close(file_fd);

			if (ret < 0) {
				print_macro(test, p->test_value,
					    p->test_name, ret, -1,
					    ENOTSUP);
			} else {
				print_macro(test, p->test_value,
					    p->test_name, ret, length, 0);
			}

			fd = open(exs_test_fifo_2, O_WRONLY);

			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			/** blocking server **/
			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_bind(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_listen(p) < 0)
				exit(EXIT_FAILURE);

			int fd = open(exs_test_fifo_1, O_WRONLY);

			if (ok_exs_accept(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_RDONLY);

			close(fd);
		}
	}

	else if (test == 15) {
		pid_t pid = fork();
		if (pid == 0) {
			/** blocking client **/
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			p->test_value = "OK_EXS_BLOCKING_SENDFILE";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			int file_fd = open(exs_test_small_file, O_RDONLY);

			struct stat fileStat;
			fstat(file_fd, &fileStat);
			size_t length = fileStat.st_size;

			sendvec.exs_xfv.exs_fdv.exs_fildes = file_fd;
			sendvec.exs_xfv.exs_fdv.exs_offset = (off_t) 0;
			sendvec.exs_xfv.exs_fdv.exs_length = (size_t) 0;	//go until EOF
			sendvec.exs_xfv.exs_fdv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_FDVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);

			ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);

			close(file_fd);

			if (ret < 0) {
				print_macro(test, p->test_value,
					    p->test_name, ret, -1,
					    ENOTSUP);
			} else {
				print_macro(test, p->test_value,
					    p->test_name, ret, length, 0);
			}

			fd = open(exs_test_fifo_2, O_WRONLY);

			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			/** blocking server **/
			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_bind(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_listen(p) < 0)
				exit(EXIT_FAILURE);

			int fd = open(exs_test_fifo_1, O_WRONLY);

			if (ok_exs_accept(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_RDONLY);

			close(fd);
		}
	}

	else if (test == 16) {
		pid_t pid = fork();
		if (pid == 0) {
			/** blocking client **/
			int fd = open(exs_test_fifo_1, O_RDONLY);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			p->test_value = "OK_EXS_BLOCKING_SENDFILE";
			p->test_name =
			    all_blocking_sendfile_tests[test - 1];

			p->file_length = 1024;

			exs_xferfile_t sendvec;
			memset(&sendvec, 0, sizeof(exs_xferfile_t));

			sendvec.exs_xfv.exs_pathv.exs_path =
			    exs_test_small_file;
			sendvec.exs_xfv.exs_pathv.exs_offset = (off_t) 0;
			sendvec.exs_xfv.exs_pathv.exs_length = (size_t) 0;	//go until EOF
			sendvec.exs_xfv.exs_pathv.exs_flags = 0;	//EXS_SHUT_WR

			p->sendvec = &sendvec;
			p->sendvec->exs_xfv_type = EXS_PATHVEC;

			p->sendvec_cnt = 1;

			int ret =
			    exs_blocking_sendfile(p->fd, p->sendvec,
						  p->sendvec_cnt,
						  p->flags);

			if (ret < 0) {
				print_macro(test, p->test_value,
					    p->test_name, ret, -1,
					    ENOTSUP);
			} else {
				print_macro(test, p->test_value,
					    p->test_name, ret,
					    p->file_length, 0);
			}

			fd = open(exs_test_fifo_2, O_WRONLY);

			close(fd);
			exit(EXIT_SUCCESS);
		} else {
			/** blocking server **/
			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_bind(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_listen(p) < 0)
				exit(EXIT_FAILURE);

			int fd = open(exs_test_fifo_1, O_WRONLY);

			if (ok_exs_accept(p) < 0)
				exit(EXIT_FAILURE);

			close(fd);

			if (ok_exs_close(p) < 0)
				exit(EXIT_FAILURE);

			fd = open(exs_test_fifo_2, O_RDONLY);

			close(fd);
		}
	}

	/** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}
}

int main(int argc, char *argv[])
{
	execute(argc, argv, test_blocking_sendfile,
		BLOCKING_SENDFILE_TESTS);
}
