/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_socket_setup.h"
#include "setup/exs_connect_setup.h"
#include "setup/exs_bind_setup.h"
#include "setup/exs_listen_setup.h"
#include "setup/exs_server_setup.h"

#define BAD_FLAG 2

void *test_listen(void *test_void_ptr)
{
	setup_fault_catcher(p, getpid());

	param *p = (param *) test_void_ptr;
	int test = p->test_number;

	/* EPERM: exs_init has not been called */
	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_listen_tests[test - 1];
		int ret = exs_listen(p->fd, 10);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EPERM);
	}

	/* EBADF: fd is invalid */
	else if (test == 2) {
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		p->test_value = "EBADF";
		p->test_name = all_listen_tests[test - 1];

		int ret = exs_listen(-1, 10);

		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EBADF);
	}

	/* EINVAL: fd already connected or listened to */
	else if (test == 3) {
		pid_t pid = fork();
		if (pid == 0) {
			p->test_value = "EINVAL";
			p->test_name = all_listen_tests[test - 1];
			int ret, fd;

			if (p->mode == 0) {
				fd = open(exs_test_fifo_1, O_RDONLY);
			}

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);
			if (ok_exs_socket(p) < 0)
				exit(EXIT_FAILURE);

			if (p->mode > 0) {
				if (ok_exs_qcreate(p) < 0)
					exit(EXIT_FAILURE);
			}

			if (ok_exs_connect(p) < 0)
				exit(EXIT_FAILURE);

			if (p->mode > 0) {
				if (ok_exs_qdequeue(p) < 0)
					exit(EXIT_FAILURE);
			}

			ret = exs_listen(p->fd, 10);

			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EINVAL);

			if (p->mode == 0) {
				close(fd);
				exs_close(p->fd, p->flags, p->qh, p->qh);
				fd = open(exs_test_fifo_2, O_WRONLY);
				close(fd);
			}
			exit(EXIT_SUCCESS);
		} else {
			exs_server(p);
		}
	}

	/* EINVAL: backlog is negative */
	else if (test == 4) {
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		p->test_value = "EINVAL";
		p->test_name = all_listen_tests[test - 1];

		int ret = exs_listen(p->fd, -1);

		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* OK_EXS_LISTEN */
	else if (test == 5) {
		p->test_value = "OK_EXS_LISTEN";
		p->test_name = all_listen_tests[test - 1];
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		if (ok_exs_socket(p) < 0)
			exit(EXIT_FAILURE);
		if (ok_exs_bind(p) < 0)
			exit(EXIT_FAILURE);

		int ret = ok_exs_listen(p);

		if (ret == 0) {
			print_macro(test, p->test_value, p->test_name, ret,
				    0, 0);
		} else {
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, ENOTSUP);
		}
	}

   /** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}
}

int main(int argc, char *argv[])
{
	execute(argc, argv, test_listen, LISTEN_TESTS);
}
